/*
 * Copyright (C) 2023  Valentin Lorentz
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License version 3,
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

use clap::{ArgMatches, Command, Parser};
use color_eyre::eyre::{anyhow, bail, Result, WrapErr};
use tokio::sync::mpsc;

use crate::action::Action;

pub type CommandHandler = fn(&str, &str, &mpsc::UnboundedSender<Action>) -> Result<()>;

pub struct RataCommand {
  pub name: &'static str,
  pub help: &'static str,
  pub handler: CommandHandler,
}

inventory::collect!(RataCommand);

/*
pub fn parser() -> Command {
  inventory::iter::<RataCommand>().fold(
    Command::new("ratatrix")
      .bin_name("")
      .subcommand_required(true),
    |parser, command| {
      if command.command.get_bin_name().is_none() {
        panic!("Command {:?} has no bin_name", command.command);
      }
      parser.subcommand(command.command.clone())
    },
  )
}*/

pub fn run_command(command_line: &str, action_tx: &mpsc::UnboundedSender<Action>) -> Result<()> {
  if command_line.bytes().nth(0) != Some(b'/') {
    bail!("Not a command: {}", command_line);
  }

  let (command_name, args) = match command_line[1..].split_once(" ") {
    Some((command_name, args)) => (command_name, args),
    None => (&command_line[1..], ""),
  };

  for command in inventory::iter::<RataCommand>() {
    if command.name == command_name.to_ascii_lowercase() {
      return (command.handler)(command_name, args, action_tx);
    }
  }

  bail!("Unknown command /{}", command_name)
}

inventory::submit! {
  RataCommand {
    name: "quit",
    help: "Exits the process",
    handler: quit_handler,
  }
}

fn quit_handler(_name: &str, _args: &str, action_tx: &mpsc::UnboundedSender<Action>) -> Result<()> {
  action_tx
    .send(Action::Quit)
    .context("Could not queue quit action")?;
  Ok(())
}

inventory::submit! {
  RataCommand {
    name: "suspend",
    help: "Puts the process in the background",
    handler: suspend_handler,
  }
}

fn suspend_handler(
  _name: &str,
  _args: &str,
  action_tx: &mpsc::UnboundedSender<Action>,
) -> Result<()> {
  action_tx
    .send(Action::Suspend)
    .context("Could not queue suspend action")?;
  Ok(())
}
